# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.2

import enum
import typing
import uuid as _uuid

import winrt.system
from winrt import _winrt_windows_ui_xaml_media_imaging

__all__ = [
    "BitmapCreateOptions",
    "DecodePixelType",
    "SvgImageSourceLoadStatus",
    "BitmapImage",
    "BitmapSource",
    "DownloadProgressEventArgs",
    "RenderTargetBitmap",
    "SoftwareBitmapSource",
    "SurfaceImageSource",
    "SvgImageSource",
    "SvgImageSourceFailedEventArgs",
    "SvgImageSourceOpenedEventArgs",
    "VirtualSurfaceImageSource",
    "WriteableBitmap",
    "XamlRenderingBackgroundTask",
    "DownloadProgressEventHandler",
]

class BitmapCreateOptions(enum.IntFlag):
    NONE = 0
    IGNORE_IMAGE_CACHE = 0x8

class DecodePixelType(enum.IntEnum):
    PHYSICAL = 0
    LOGICAL = 1

class SvgImageSourceLoadStatus(enum.IntEnum):
    SUCCESS = 0
    NETWORK_ERROR = 1
    INVALID_FORMAT = 2
    OTHER = 3

BitmapImage = _winrt_windows_ui_xaml_media_imaging.BitmapImage
BitmapSource = _winrt_windows_ui_xaml_media_imaging.BitmapSource
DownloadProgressEventArgs = _winrt_windows_ui_xaml_media_imaging.DownloadProgressEventArgs
RenderTargetBitmap = _winrt_windows_ui_xaml_media_imaging.RenderTargetBitmap
SoftwareBitmapSource = _winrt_windows_ui_xaml_media_imaging.SoftwareBitmapSource
SurfaceImageSource = _winrt_windows_ui_xaml_media_imaging.SurfaceImageSource
SvgImageSource = _winrt_windows_ui_xaml_media_imaging.SvgImageSource
SvgImageSourceFailedEventArgs = _winrt_windows_ui_xaml_media_imaging.SvgImageSourceFailedEventArgs
SvgImageSourceOpenedEventArgs = _winrt_windows_ui_xaml_media_imaging.SvgImageSourceOpenedEventArgs
VirtualSurfaceImageSource = _winrt_windows_ui_xaml_media_imaging.VirtualSurfaceImageSource
WriteableBitmap = _winrt_windows_ui_xaml_media_imaging.WriteableBitmap
XamlRenderingBackgroundTask = _winrt_windows_ui_xaml_media_imaging.XamlRenderingBackgroundTask
DownloadProgressEventHandler = typing.Callable[[typing.Optional[winrt.system.Object], typing.Optional[DownloadProgressEventArgs]], None]
